/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.robot.ev3;

import org.jabotics.robot.defs.*;
import lejos.hardware.ev3.LocalEV3;
import lejos.hardware.lcd.TextLCD;

/**
 * ADisplay is the implementation class for the interface IDisplayAdapter on the EV3 robot.
 * 
 * @author  Frank Gollnik
 * @version 1.1.2
 */
public class ADisplay implements IDisplayAdapter
{
    static private final IDisplayAdapter sDisplayAdapter = new ADisplay();
    static private final TextLCD mLCD = LocalEV3.get().getTextLCD();

    static IDisplayAdapter getAdapter()
    {
        return sDisplayAdapter;
    }

    private ADisplay()
    {
    }

    @Override
    public void refresh()
    {
        mLCD.refresh();
    }
    
    @Override
    public void clear()
    {
        mLCD.clear();
    }
    
    @Override
    public void clear(int y_position)
    {
        mLCD.clear(y_position);
    }
    
    @Override
    public void clear(int x_position, int y_position, int count)
    {
        mLCD.clear(x_position, y_position, count);
    }
    
    @Override
    public void draw(char ch, int x_position, int y_position)
    {
        mLCD.drawChar(ch, x_position, y_position);
    }
    
    @Override
    public void draw(int n, int x_position, int y_position)
    {
        mLCD.drawInt(n, x_position, y_position);
    }
    
    @Override
    public void draw(int n, int x_position, int y_position, int digits)
    {
        mLCD.drawInt(n, digits, x_position, y_position); // different ordering of parameters!
    }
    
    @Override
    public void draw(float fl, int x_position, int y_position)
    {
        mLCD.drawString(Float.toString(fl), x_position, y_position);
    }
    
    @Override
    public void draw(String text, int x_position, int y_position)
    {
        mLCD.drawString(text, x_position, y_position);
    }
    
    @Override
    public void draw(String text, int x_position, int y_position, boolean inverted)
    {
        mLCD.drawString(text, x_position, y_position, inverted);
    }
}
