/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.robot.ev3;

import org.jabotics.robot.defs.*;
import lejos.hardware.Sound;

/**
 * ASound is the implementation class for the interface ISoundAdapter on the EV3 robot.
 * 
 * @author  Frank Gollnik
 * @version 1.1.2
 */
public class ASound implements ISoundAdapter
{
    private static final ISoundAdapter sSoundAdapter = new ASound();

    static ISoundAdapter getAdapter()
    {
        return sSoundAdapter;
    }

    private ASound()
    {
    }

    @Override
    public int getVolume()
    {
        return Sound.getVolume();
    }

    @Override
    public void setVolume(int volume)
    {
        Sound.setVolume(volume);
    }

    @Override
    public void beep()
    {
        Sound.beep();
    }

    @Override
    public void twoBeeps()
    {
        Sound.twoBeeps();
    }

    @Override
    public void beepSequence()
    {
        Sound.beepSequence();
    }

    @Override
    public void beepSequenceUp()
    {
        Sound.beepSequenceUp();
    }

    @Override
    public void buzz()
    {
        Sound.buzz();
    }

    @Override
    public void tone(int frequency, int time)
    {
        Sound.playTone(frequency, time);
    }

    @Override
    public void tone(int frequency, int time, int volume)
    {
        Sound.playTone(frequency, time, volume);
    }

    @Override
    public void note(ESound instrument, int frequency, int time)
    {
        // SOUND MODULATION DEACTIVATED FOR PERFORMANCE REASONS
        tone(frequency, time);
        /*switch (instrument)
        {
            case PIANO:
            {
                Sound.playNote(Sound.PIANO, frequency, time);
                break;
            }
            case FLUTE:
            {
                Sound.playNote(Sound.FLUTE, frequency, time);
                break;
            }
            default:
            {
                Sound.playNote(Sound.XYLOPHONE, frequency, time);
                break;
            }
        }*/
    }

    @Override
    public void pause(int time)
    {
        Sound.pause(time);
    }
}
