/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.robot.nxt;

import org.jabotics.robot.defs.*;
import lejos.nxt.Battery;
import lejos.nxt.MotorPort;
import lejos.nxt.TachoMotorPort;


/**
 * AMotor is the implementation class for the interface IMotorAdapter on the NXT robot.
 * 
 * @author  Frank Gollnik
 * @version 1.1.2
 */
public class AMotor implements IMotorAdapter
{
    static private AMotor[] sMotors = null;

    // Instanzvariablen:
    private MotorPort mMotorPort;
    private volatile float mTachoCountCorrectionFactor;

    static public synchronized AMotor getMotor(EMotorPort port)
    {
        if (null == sMotors) sMotors = new AMotor[] {new AMotor(MotorPort.A),new AMotor(MotorPort.B),new AMotor(MotorPort.C)};

        return sMotors[port.ordinal()];
    }

    private AMotor(MotorPort motorPort)
    {
        mMotorPort = motorPort;
        mTachoCountCorrectionFactor = 1f;
    }

    @Override
    public float getMaxSpeed()
    {
        return Battery.getVoltage() * 100.0f;
    }

    @Override
    public void startControl()
    {
        // Die Klasse MotorPort erkennt an dem Wert ihrer Instanzvariablen _PWMMode, ob ein Motor verwendet wird.
        // Der Wert PWM_BRAKE bedeutet: Motor wird aus dem Programm verwendet; der Wert PWM_FLOAT (Default) bedeutet: nicht verwendet
        mMotorPort.setPWMMode(TachoMotorPort.PWM_BRAKE);
    }

    @Override
    public void setToFloatMode()
    {
        mMotorPort.controlMotor(0,TachoMotorPort.FLOAT);
    }

    @Override
    public void setPower(float power)
    {
        int powerInt = Math.round(power);
        mMotorPort.controlMotor(powerInt, (0 == powerInt) ? TachoMotorPort.STOP : TachoMotorPort.FORWARD);
    }

    @Override
    public float getTachoCount()
    {
        return (mTachoCountCorrectionFactor * (float)mMotorPort.getTachoCount());
    }

    @Override
    public float[] getPIDParameters()
    {
        return new float[] {6f,0.04f,22f,2f,0.04f,8f}; // P (control), I (control),D  (control), P (halted), I (halted), D (halted)
    }

    @Override
    public void setTachoCountCorrectionFactor(float correctionFactor)
    {
        mTachoCountCorrectionFactor = (correctionFactor < 0.9f) ? 0.9f : ((correctionFactor > 1.1f) ? 1.1f : correctionFactor);
    }

    @Override
    public void freezeAllTachoCounts()
    {
    }

    @Override
    public void releaseAllTachoCounts()
    {
    }

    @Override
    public void commitAllPowerData()
    {
    }
}
