/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.robot.en;


/**
 * <font color="LimeGreen"><b>&bull;</b></font> The interface ILightSensor extends the interface {@link ISensor}
 * by methods for controlling the LED of the light sensor.
 * 
 * @author  Frank Gollnik
 * @version 1.1.2
 */

public interface ILightSensor extends ISensor
{
    /**
     * <font color="LimeGreen"><b>&bull;</b></font> Switches on the LED of the light sensor.
     * For the EV3 robot, the light sensor is sensitive to red light only, if the LED is switched on.
     * The light sensor therefore will register mainly the reflected light from the LED.
     * With the LED turned off, the light sensor of the EV3 will register the complete ambient light.
     * Since a change of the measuring mode is accompanied by an initialization procedure,
     * turning on/off the LED results in a certain time delay on the EV3 robot.
     */
    public void switchLightOn();

    /**
     * <font color="LimeGreen"><b>&bull;</b></font> Switches off the LED of the light sensor.
     */
    public void switchLightOff();
}
