/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.robot.sim;

import org.jabotics.robot.defs.*;

/**
 * Beschreiben Sie hier die Klasse SIMAdapter.
 * 
 * @author  Frank Gollnik
 * @version 1.1.2
 */
public class SIMAdapter implements IAdapterFactory, ITimerAdapter
{
    // SIMAdapter is a possible entry point into the simulation.
    // => proper initialization of simulation model needs to be secured during class initialization.
    // => can be achieved by triggering class initialization of XSession:
    static final XSession sSession = XSession.getSession();
    static final SIMAdapter sSIMAdapter = new SIMAdapter();

    static
    {
        org.jabotics.robot.adapter.XAdapterFactoryHandle.init(sSIMAdapter);
    }

    
    private SIMAdapter()
    {
    }

    ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    // methods of IAdapterFactory
    ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

    @Override
    public IMotorAdapter getMotorAdapter(EMotorPort port)
    {
        XSimulation.start();
        return XMotor.getMotor(port);
    }

    @Override
    public ISensorAdapter getSensorAdapter(ESensorPort port, ESensorType type)
    {
        XSimulation.start();
        return XSensorRef.getExternalRef(port,type);
    }

    @Override
    public ISensorAdapter getButtonAdapter(EButtonType type)
    {
        XSimulation.start();
        return XSensorRef.getExternalRef(type);
    }

    @Override
    public ISensorAdapter getInvalidSensorAdapter()
    {
        XSimulation.start();
        return XSensorRef.getInvalidRef();
    }

    @Override
    public ISoundAdapter getSoundAdapter()
    {
        XSimulation.start();
        return XSpeaker.getSpeaker();
    }

    @Override
    public IDisplayAdapter getDisplayAdapter()
    {
        XSimulation.start();
        return XDisplay.getDisplay();
    }

    @Override
    public ITimerAdapter getTimerAdapter()
    {
        XSimulation.start();
        return this;
    }

    @Override
    public IConsoleAdapter getConsoleAdapter()
    {
        XSimulation.start();
        return XConsole.getConsole();
    }

    @Override
    public ISessionAdapter getSessionAdapter()
    {
        return sSession;
    }

    @Override
    public IRobotAdapter getRobotAdapter()
    {
        XSimulation.start();
        return XSimulationModel.getRobot();
    }


    ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    // methods of ITimerAdapter
    ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

    @Override
    public long currentTimeMillis()
    {
        return XTimer.currentTimeMillisRobot();
    }

    @Override
    public void msDelay(long period)
    {
        XTimer.msDelayRobot(period);
    }
}
