/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.robot.sim;

import java.awt.Rectangle;
import java.awt.geom.Point2D;
import java.awt.geom.Path2D;
import java.util.ArrayList;

/**
 * Beschreiben Sie hier die Klasse XScene.
 * 
 * @author  Frank Gollnik
 * @version 1.1.2
 */
public class XScene extends BModelElement
{
    static final Rectangle.Float BOUNDS = new Rectangle.Float(-20f,-20f,840f,640f);

    private final String mSceneID = "a09e6a33-1152-4306-90ff-d7c36145df68";
    private final float mDefaultRobotStartX = 600f;
    private final float mDefaultRobotStartY = 400f;
    private final float mDefaultRobotStartPhi = 0f;
    private ArrayList<Point2D.Float> mOuterCorners = new ArrayList<Point2D.Float>(4);
    private ArrayList<Point2D.Float> mInnerCorners = new ArrayList<Point2D.Float>(26);
    private Path2D.Float mPath = new Path2D.Float(Path2D.WIND_EVEN_ODD, 30);
    private XFloor mFloor;
    private XWall mWall;

    XScene()
    {
        super();

        mOuterCorners.add(new Point2D.Float(820f,620f));   // P1
        mOuterCorners.add(new Point2D.Float(820f,-20f));   // P2
        mOuterCorners.add(new Point2D.Float(-20f,-20f));   // P3
        mOuterCorners.add(new Point2D.Float(-20f,620f));   // P4
        
        mInnerCorners.add(new Point2D.Float(800f,600f));   // P5
        mInnerCorners.add(new Point2D.Float(800f,0f));     // P6
        mInnerCorners.add(new Point2D.Float(0f,0f));       // P7
        mInnerCorners.add(new Point2D.Float(0f,240f));     // P8
        mInnerCorners.add(new Point2D.Float(460f,240f));   // P9
        mInnerCorners.add(new Point2D.Float(460f,380f));   // P10
        mInnerCorners.add(new Point2D.Float(340f,380f));   // P11
        mInnerCorners.add(new Point2D.Float(340f,500f));   // P12
        mInnerCorners.add(new Point2D.Float(320f,500f));   // P13
        mInnerCorners.add(new Point2D.Float(320f,380f));   // P14
        mInnerCorners.add(new Point2D.Float(100f,380f));   // P15
        mInnerCorners.add(new Point2D.Float(100f,360f));   // P16
        mInnerCorners.add(new Point2D.Float(440f,360f));   // P17
        mInnerCorners.add(new Point2D.Float(440f,260f));   // P18
        mInnerCorners.add(new Point2D.Float(0f,260f));     // P19
        mInnerCorners.add(new Point2D.Float(0f,600f));     // P20
        mInnerCorners.add(new Point2D.Float(200f,600f));   // P21
        mInnerCorners.add(new Point2D.Float(200f,500f));   // P22
        mInnerCorners.add(new Point2D.Float(100f,500f));   // P23
        mInnerCorners.add(new Point2D.Float(100f,480f));   // P24
        mInnerCorners.add(new Point2D.Float(220f,480f));   // P25
        mInnerCorners.add(new Point2D.Float(220f,600f));   // P26
        mInnerCorners.add(new Point2D.Float(440f,600f));   // P27
        mInnerCorners.add(new Point2D.Float(440f,480f));   // P28
        mInnerCorners.add(new Point2D.Float(460f,480f));   // P29
        mInnerCorners.add(new Point2D.Float(460f,600f));   // P30

        int sizeInnerCorners = mInnerCorners.size();
        int sizeOuterCorners = mOuterCorners.size();

        Point2D.Float pt = mInnerCorners.get(0);
        mPath.moveTo(pt.x,pt.y);
        for (int i = 1; i < sizeInnerCorners; ++i)
        {
            pt = mInnerCorners.get(i);
            mPath.lineTo(pt.x,pt.y);
        }
        mPath.closePath();

        pt = mOuterCorners.get(0);
        mPath.moveTo(pt.x,pt.y);
        for (int i = 1; i < sizeOuterCorners; ++i)
        {
            pt = mOuterCorners.get(i);
            mPath.lineTo(pt.x,pt.y);
        }
        mPath.closePath();

        mFloor = new XFloor();
        addChild(mFloor);
        mWall = new XWall(mPath);
        addChild(mWall);
    }

    Path2D.Float getWallPath()
    {
        return mPath;
    }

    XFloor getFloor()
    {
        return mFloor;
    }

    XWall getWall()
    {
        return mWall;
    }

    @Override
    public void manageVisitor(BModelVisitor visitor)
    {
        visitor.visitScene(this);
    }

    public String getID()
    {
        return mSceneID;
    }

    public float getDefaultRobotStartX()
    {
        return mDefaultRobotStartX;
    }

    public float getDefaultRobotStartY()
    {
        return mDefaultRobotStartY;
    }

    public float getDefaultRobotStartPhi()
    {
        return mDefaultRobotStartPhi;
    }

    public static void printRandomUUID()
    {
        System.out.println(java.util.UUID.randomUUID().toString());
    }
}
