/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.robot.sim;

import org.jabotics.robot.defs.*;
import java.util.Locale;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;

/**
 * Beschreiben Sie hier die Klasse XSettingsPanel.
 * 
 * @author  Frank Gollnik
 * @version 1.1.2
 */
@SuppressWarnings("unchecked")
class XSettingsPanel extends JPanel implements XSimulation.IListener, XPreferences.IListener, XSensorRef.IListener, ActionListener
{
    private final boolean mIsInteractive;
    private final XRobot mRobot;

    private XPreferences.Read mPrefs = null;
    private Locale mLocale = null;
    private boolean mIsSettingPreferences = false;

    private final String[] mSensorTypes = {XMsg.NO_SENSOR,XMsg.TOUCH_SENSOR,XMsg.SOUND_SENSOR,XMsg.LIGHT_SENSOR,XMsg.ULTRASONIC_SENSOR};
    private final String[] mSensorDirections = {XMsg.NO_DIRECTION,XMsg.FRONT,XMsg.BACK,XMsg.DOWNWARD,XMsg.FORWARD,XMsg.BACKWARD,XMsg.TO_LEFT,XMsg.TO_RIGHT};
    private final XSensorRef[] mInternalSensorRefs = new XSensorRef[D.SENSOR_PORT_NUMBER];

    private final JCheckBox mChBSimBreaksEnabled;
    private final JCheckBox mChBSimSuspendEnabled;
    private final JComboBox<String>[] mCoBSensorType;
    private final JComboBox<String>[] mCoBDirection;
    private final JComboBox<XAudioSystem.CBMixerInfo> mCoBMixerIn;
    private final JComboBox<XAudioSystem.CBMixerInfo> mCoBMixerOut;
    private final JTextField mTxtRobotStartX;
    private final JTextField mTxtRobotStartY;
    private final JTextField mTxtRobotStartPhi;
    private final JButton mBtnDefault;
    private final JButton mBtnImport;
    private final JButton mBtnExport;

    XSettingsPanel(boolean interactive)
    {
        super(new GridBagLayout());

        mIsInteractive = interactive;
        mRobot = XSimulationModel.getRobot();

        GridBagConstraints c;
        Insets smallMargin = new Insets(1,1,1,1);

        JPanel paneBreaks = new JPanel(new GridBagLayout());
        TitledBorder tb1 = new TitledBorder(XMsg.SIM_BREAKS);
        tb1.setTitleColor(Color.gray);
        tb1.setBorder(new LineBorder(Color.gray,1,true));
        paneBreaks.setBorder(tb1);
        c = new GridBagConstraints();
        c.gridx = 0;
        c.gridy = 0;
        c.weightx = 1.0;
        c.insets = new Insets(3,1,1,1);
        c.fill = GridBagConstraints.HORIZONTAL;
        this.add(paneBreaks,c);

        mChBSimBreaksEnabled = new JCheckBox(XMsg.ENABLE_SIM_BREAKS);
        mChBSimBreaksEnabled.setToolTipText(XMsg.ENABLE_SIM_BREAKS_INFO);
        c = new GridBagConstraints();
        c.gridx = 0;
        c.gridy = 0;
        c.weightx = 1.0;
        c.fill = GridBagConstraints.HORIZONTAL;
        paneBreaks.add(mChBSimBreaksEnabled,c);

        JPanel paneThread = new JPanel(new GridBagLayout());
        c = new GridBagConstraints();
        c.gridx = 0;
        c.gridy = 0;
        c.weightx = 0.0;
        c.fill = GridBagConstraints.NONE;
        paneThread.add(new JLabel("     "),c);
        c = new GridBagConstraints();
        c.gridx = 1;
        c.gridy = 0;
        c.weightx = 1.0;
        c.fill = GridBagConstraints.HORIZONTAL;
        mChBSimSuspendEnabled = new JCheckBox(XMsg.SUSPEND_ROBOT_THREAD);
        mChBSimSuspendEnabled.setToolTipText(XMsg.SUSPEND_ROBOT_THREAD_INFO);
        paneThread.add(mChBSimSuspendEnabled,c);
        c = new GridBagConstraints();
        c.gridx = 0;
        c.gridy = 1;
        c.weightx = 1.0;
        c.fill = GridBagConstraints.HORIZONTAL;
        paneBreaks.add(paneThread,c);

        JPanel paneSensors = new JPanel(new GridBagLayout());
        TitledBorder tb2 = new TitledBorder(XMsg.SENSORS);
        tb2.setTitleColor(Color.gray);
        tb2.setBorder(new LineBorder(Color.gray,1,true));
        paneSensors.setBorder(tb2);
        c = new GridBagConstraints();
        c.gridx = 0;
        c.gridy = 1;
        c.weightx = 1.0;
        c.insets = smallMargin;
        c.fill = GridBagConstraints.HORIZONTAL;
        this.add(paneSensors,c);

        mCoBSensorType = new JComboBox[D.SENSOR_PORT_NUMBER];
        mCoBDirection = new JComboBox[D.SENSOR_PORT_NUMBER];
        for (int i = 0;  i < D.SENSOR_PORT_NUMBER;  ++i)
        {
            c = new GridBagConstraints();
            c.gridx = 0;
            c.gridy = i;
            c.insets = smallMargin;
            c.fill = GridBagConstraints.NONE;
            //paneSensors.add(new JLabel(XMsg.PORT + " " + (i+1) + ": "),c);
            paneSensors.add(new JLabel("  " + String.valueOf(i+1) + ":  "),c);

            mCoBSensorType[i] = new JComboBox<String>(mSensorTypes);
            c = new GridBagConstraints();
            c.gridx = 1;
            c.gridy = i;
            c.weightx = 1.0;
            c.insets = smallMargin;
            c.fill = GridBagConstraints.HORIZONTAL;
            paneSensors.add(mCoBSensorType[i],c);

            mCoBDirection[i] = new JComboBox<String>(mSensorDirections);
            c = new GridBagConstraints();
            c.gridx = 2;
            c.gridy = i;
            c.weightx = 1.0;
            c.insets = smallMargin;
            c.fill = GridBagConstraints.HORIZONTAL;
            paneSensors.add(mCoBDirection[i],c);
        }


        JPanel paneMixer = new JPanel(new GridBagLayout());
        TitledBorder tb3 = new TitledBorder(XMsg.AUDIO_SYSTEM);
        tb3.setTitleColor(Color.gray);
        tb3.setBorder(new LineBorder(Color.gray,1,true));
        paneMixer.setBorder(tb3);
        c = new GridBagConstraints();
        c.gridx = 0;
        c.gridy = 2;
        c.weightx = 1.0;
        c.insets = smallMargin;
        c.fill = GridBagConstraints.HORIZONTAL;
        this.add(paneMixer,c);
    
        c = new GridBagConstraints();
        c.gridx = 0;
        c.gridy = 0;
        c.insets = smallMargin;
        c.fill = GridBagConstraints.NONE;
        c.anchor = GridBagConstraints.EAST;
        paneMixer.add(new JLabel(XMsg.AUDIO_IN + ": "),c);

        mCoBMixerIn = new JComboBox<XAudioSystem.CBMixerInfo>();
        c = new GridBagConstraints();
        c.gridx = 1;
        c.gridy = 0;
        c.weightx = 1.0;
        c.insets = smallMargin;
        c.fill = GridBagConstraints.HORIZONTAL;
        paneMixer.add(mCoBMixerIn,c);

        c = new GridBagConstraints();
        c.gridx = 0;
        c.gridy = 1;
        c.insets = smallMargin;
        c.fill = GridBagConstraints.NONE;
        c.anchor = GridBagConstraints.EAST;
        paneMixer.add(new JLabel(XMsg.AUDIO_OUT + ": "),c);

        mCoBMixerOut = new JComboBox<XAudioSystem.CBMixerInfo>();
        c = new GridBagConstraints();
        c.gridx = 1;
        c.gridy = 1;
        c.weightx = 1.0;
        c.insets = smallMargin;
        c.fill = GridBagConstraints.HORIZONTAL;
        paneMixer.add(mCoBMixerOut,c);


        JPanel panePosition = new JPanel(new GridBagLayout());
        TitledBorder tb4 = new TitledBorder(XMsg.START_POSITION);
        tb4.setTitleColor(Color.gray);
        tb4.setBorder(new LineBorder(Color.gray,1,true));
        panePosition.setBorder(tb4);
        panePosition.setToolTipText(XMsg.START_INFO);
        c = new GridBagConstraints();
        c.gridx = 0;
        c.gridy = 3;
        c.weightx = 1.0;
        c.insets = smallMargin;
        c.fill = GridBagConstraints.HORIZONTAL;
        this.add(panePosition,c);

        c = new GridBagConstraints();
        c.gridx = 0;
        c.gridy = 0;
        c.insets = smallMargin;
        c.fill = GridBagConstraints.NONE;
        panePosition.add(new JLabel("X: "),c);

        mTxtRobotStartX = new JTextField(8);
        mTxtRobotStartX.setHorizontalAlignment(JTextField.RIGHT); 
        mTxtRobotStartX.setEditable(false);
        mTxtRobotStartX.setToolTipText(XMsg.START_INFO);
        c = new GridBagConstraints();
        c.gridx = 1;
        c.gridy = 0;
        c.weightx = 1.0;
        c.insets = smallMargin;
        c.fill = GridBagConstraints.HORIZONTAL;
        panePosition.add(mTxtRobotStartX,c);

        c = new GridBagConstraints();
        c.gridx = 2;
        c.gridy = 0;
        c.insets = smallMargin;
        c.fill = GridBagConstraints.NONE;
        panePosition.add(new JLabel("   Y: "),c);

        mTxtRobotStartY = new JTextField(8);
        mTxtRobotStartY.setHorizontalAlignment(JTextField.RIGHT); 
        mTxtRobotStartY.setEditable(false);
        mTxtRobotStartY.setToolTipText(XMsg.START_INFO);
        c = new GridBagConstraints();
        c.gridx = 3;
        c.gridy = 0;
        c.weightx = 1.0;
        c.insets = smallMargin;
        c.fill = GridBagConstraints.HORIZONTAL;
        panePosition.add(mTxtRobotStartY,c);

        c = new GridBagConstraints();
        c.gridx = 4;
        c.gridy = 0;
        c.insets = smallMargin;
        c.fill = GridBagConstraints.NONE;
        panePosition.add(new JLabel("   O: "),c);
        
        mTxtRobotStartPhi = new JTextField(8);
        mTxtRobotStartPhi.setHorizontalAlignment(JTextField.RIGHT); 
        mTxtRobotStartPhi.setEditable(false);
        mTxtRobotStartPhi.setToolTipText(XMsg.START_INFO);
        c = new GridBagConstraints();
        c.gridx = 5;
        c.gridy = 0;
        c.weightx = 1.0;
        c.insets = smallMargin;
        c.fill = GridBagConstraints.HORIZONTAL;
        panePosition.add(mTxtRobotStartPhi,c);

        JPanel paneButtons = new JPanel(new GridBagLayout());
        c = new GridBagConstraints();
        c.gridx = 0;
        c.gridy = 4;
        c.weightx = 1.0;
        c.weighty = 1.0;
        c.insets = smallMargin;
        //c.anchor = GridBagConstraints.NORTH;
        c.anchor = GridBagConstraints.SOUTH;
        c.fill = GridBagConstraints.HORIZONTAL;
        this.add(paneButtons,c);

        mBtnDefault = new JButton(XMsg.SET_DEFAULT);
        c = new GridBagConstraints();
        c.gridx = 0;
        c.gridy = 0;
        c.weightx = 1.0;
        c.insets = smallMargin;
        c.fill = GridBagConstraints.HORIZONTAL;
        paneButtons.add(mBtnDefault,c);

        mBtnImport = new JButton(XMsg.IMPORT);
        c = new GridBagConstraints();
        c.gridx = 1;
        c.gridy = 0;
        c.weightx = 1.0;
        c.insets = smallMargin;
        c.fill = GridBagConstraints.HORIZONTAL;
        paneButtons.add(mBtnImport,c);

        mBtnExport = new JButton(XMsg.EXPORT);
        c = new GridBagConstraints();
        c.gridx = 2;
        c.gridy = 0;
        c.weightx = 1.0;
        c.insets = smallMargin;
        c.fill = GridBagConstraints.HORIZONTAL;
        paneButtons.add(mBtnExport,c);
    }

    void init()
    {
        init(XSimulation.getLocale());
    }

    private void init(Locale locale)
    {
        if (null != mPrefs) return;

        mPrefs = XPreferences.getReadInstance();

        mLocale = (null == locale) ? Locale.getDefault() : locale;

        mChBSimBreaksEnabled.setSelected(mPrefs.getSimBreaksEnabled());
        mChBSimBreaksEnabled.setEnabled(mIsInteractive);

        mChBSimSuspendEnabled.setSelected(mPrefs.getSimSuspendEnabled());
        mChBSimSuspendEnabled.setEnabled(mIsInteractive);

        for (ESensorPort port : ESensorPort.values())
        {
            int i = port.ordinal();
            mInternalSensorRefs[i] = XSensorRef.getInternalRef(port);
            ESensorType type = mPrefs.getSensorType(port);
            ESensorDirection direction = mPrefs.getSensorDirection(port);
            setSensorType(i, type);
            mCoBDirection[i].setSelectedItem(mSensorDirections[direction.ordinal()]);
            if (mIsInteractive)
            {
                // check if sensor type already has been "fixed" due to prior call to XSimulation.getSensor(...):
                if (XSensorRef.hasExternalRefs(port))
                {
                    mCoBSensorType[i].setEnabled(false);
                }
            }
            else
            {
                mCoBSensorType[i].setEnabled(false);
                mCoBDirection[i].setEnabled(false);
            }
        }

        XAudioSystem.CBMixerInfo[] mixerInfosIn = XAudioSystem.getCBMixerInfosIn();
        for (XAudioSystem.CBMixerInfo mixerInfo : mixerInfosIn)
        {
            mCoBMixerIn.addItem(mixerInfo);
        }
        mCoBMixerIn.setSelectedItem(XAudioSystem.getCBMixerInfo(mPrefs.getAudioMixerIn()));
        mCoBMixerIn.setEnabled(mIsInteractive);

        XAudioSystem.CBMixerInfo[] mixerInfosOut = XAudioSystem.getCBMixerInfosOut();
        for (XAudioSystem.CBMixerInfo mixerInfo : mixerInfosOut)
        {
            mCoBMixerOut.addItem(mixerInfo);
        }
        mCoBMixerOut.setSelectedItem(XAudioSystem.getCBMixerInfo(mPrefs.getAudioMixerOut()));
        mCoBMixerIn.setEnabled(mIsInteractive);

        XScene scene = XSimulationModel.getScene();
        // Simulation's coordinate system: x to right, y downward, phi clockwise, origin at upper left corner of scenery
        // Robot's coordinate system: x to right, y upward, phi counter-clockwise, origin at scene's default start position
        // (see comments in XRobot for details)
        mTxtRobotStartX.setText(String.format(mLocale, "%.1f", mPrefs.getRobotStartX() - scene.getDefaultRobotStartX()));
        mTxtRobotStartY.setText(String.format(mLocale, "%.1f", scene.getDefaultRobotStartY() - mPrefs.getRobotStartY())); // y downward to y upward
        mTxtRobotStartPhi.setText(String.format(mLocale, "%.1f", calculateDegrees(scene.getDefaultRobotStartPhi() - mPrefs.getRobotStartPhi())));  // clockwise to counter-clockwise

        mBtnDefault.setEnabled(mIsInteractive);
        mBtnImport.setEnabled(mIsInteractive);
        mBtnExport.setEnabled(mIsInteractive);

        if (mIsInteractive)
        {
            mChBSimBreaksEnabled.addActionListener(this);
            mChBSimSuspendEnabled.addActionListener(this);
            for (int i = 0;  i < D.SENSOR_PORT_NUMBER;  ++i)
            {
                mCoBSensorType[i].addActionListener(this);
                mCoBDirection[i].addActionListener(this);
            }
            mCoBMixerIn.addActionListener(this);
            mCoBMixerOut.addActionListener(this);
            mBtnDefault.addActionListener(this);
            mBtnImport.addActionListener(this);
            mBtnExport.addActionListener(this);

            XSimulation.addListener(this);
        }

        XPreferences.addListener(this);
        XSensorRef.addListener(this);
    }

    private void setSensorType(int port_id, ESensorType type)
    {
        mCoBSensorType[port_id].setSelectedIndex(type.ordinal());
        initCoBDirections(port_id, type);
    }

    private void initCoBDirections(int port_id, ESensorType type)
    {
        mCoBDirection[port_id].removeAllItems();
        ESensorDirection[] directions = BSensor.getSensorDirections(type);
        for (int i = 0;  i < directions.length;  ++i)
        {
            mCoBDirection[port_id].addItem(mSensorDirections[directions[i].ordinal()]);
        }
    }

    @Override
    public void stateChanged(XSimulation.EState oldState, XSimulation.EState newState)
    {
        // registered as listener only if mIsInteractive == true
        boolean not_running = (XSimulation.EState.RUNNING != newState);
        mBtnDefault.setEnabled(not_running);
        mBtnImport.setEnabled(not_running);
        mBtnExport.setEnabled(not_running);
    }

    @Override
    public void sensorRefChanged(ESensorPort port)
    {
        if (mIsSettingPreferences) return;

        int portID = port.ordinal();
        if ((0 > portID) || (D.SENSOR_PORT_NUMBER <= portID)) return;

        JComboBox<String> cbt = mCoBSensorType[portID];
        JComboBox<String> cbd = mCoBDirection[portID];
        XSensorRef ref = mInternalSensorRefs[portID];
        ESensorType type = ref.getType();
        if (!mSensorTypes[type.ordinal()].equals(cbt.getSelectedItem()))
        {
            cbt.removeActionListener(this);
            cbd.removeActionListener(this);
            setSensorType(portID,type);
            cbd.addActionListener(this);
            cbt.addActionListener(this);
        }
        ESensorDirection direction = ref.getDirection();
        if (!mSensorDirections[direction.ordinal()].equals(cbd.getSelectedItem()))
        {
            cbd.removeActionListener(this);
            cbd.setSelectedItem(mSensorDirections[direction.ordinal()]);
            cbd.addActionListener(this);
        }
        if (mIsInteractive) cbt.setEnabled(!ref.sensorHasExternalRefs());
        cbd.setForeground(ref.isDisplayed() ? Color.black : Color.gray);
        cbd.setToolTipText(ref.isDisplayed() ? null : XMsg.SENSOR_NOT_DISPLAYED);
        this.requestFocusInWindow();
    }

    @Override
    public void preferencesChanged(XPreferences.Read newPrefs)
    {
        if (mIsSettingPreferences) return;

        if (null == mPrefs) init(XSimulation.getLocale());

        if (mPrefs.getSimBreaksEnabled() != newPrefs.getSimBreaksEnabled())
        {
            mChBSimBreaksEnabled.removeActionListener(this);
            mChBSimBreaksEnabled.setSelected(newPrefs.getSimBreaksEnabled());
            mChBSimBreaksEnabled.addActionListener(this);
        }

        if (mPrefs.getSimSuspendEnabled() != newPrefs.getSimSuspendEnabled())
        {
            mChBSimSuspendEnabled.removeActionListener(this);
            mChBSimSuspendEnabled.setSelected(newPrefs.getSimSuspendEnabled());
            mChBSimSuspendEnabled.addActionListener(this);
        }

        // Update of sensor settings via sensorRefChanged(ESensorPort port):
        // XSensorRef has its own preferences listener that will send all necessary notifications via sensorRefChanged

        if (mPrefs.getAudioMixerIn() != newPrefs.getAudioMixerIn())
        {
            mCoBMixerIn.removeActionListener(this);
            mCoBMixerIn.setSelectedItem(XAudioSystem.getCBMixerInfo(newPrefs.getAudioMixerIn()));
            mCoBMixerIn.addActionListener(this);
        }

        if (mPrefs.getAudioMixerOut() != newPrefs.getAudioMixerOut())
        {
            mCoBMixerOut.removeActionListener(this);
            mCoBMixerOut.setSelectedItem(XAudioSystem.getCBMixerInfo(newPrefs.getAudioMixerOut()));
            mCoBMixerOut.addActionListener(this);
        }

        if ( (mPrefs.getRobotStartX() != newPrefs.getRobotStartX())
          || (mPrefs.getRobotStartY() != newPrefs.getRobotStartY())
          || (mPrefs.getRobotStartPhi() != newPrefs.getRobotStartPhi()) )
        {
            XScene scene = XSimulationModel.getScene();
            // Simulation's coordinate system: x to right, y downward, phi clockwise, origin at upper left corner of scenery
            // Robot's coordinate system: x to right, y upward, phi counter-clockwise, origin at scene's default start position
            // (see comments in XRobot for details)
            mRobot.setPositionAndOrientation(newPrefs.getRobotStartX(), newPrefs.getRobotStartY(), newPrefs.getRobotStartPhi());
            mTxtRobotStartX.setText(String.format(mLocale, "%.1f", newPrefs.getRobotStartX() - scene.getDefaultRobotStartX()));
            mTxtRobotStartY.setText(String.format(mLocale, "%.1f", scene.getDefaultRobotStartY() - newPrefs.getRobotStartY())); // y downward to y upward
            mTxtRobotStartPhi.setText(String.format(mLocale, "%.1f", calculateDegrees(scene.getDefaultRobotStartPhi() - newPrefs.getRobotStartPhi())));  // clockwise to counter-clockwise
        }

        mPrefs = newPrefs;
    }

    public void actionPerformed(ActionEvent e)
    {
        Object source = e.getSource();

        if (source instanceof JCheckBox)
        {
            if ((JCheckBox)source == mChBSimBreaksEnabled)
            {
                XPreferences.Write pw = XPreferences.getWriteInstance();
                pw.setSimBreaksEnabled(mChBSimBreaksEnabled.isSelected());
                mChBSimSuspendEnabled.setEnabled(pw.getSimBreaksEnabled());
                storePreferences(pw);
                return;
            }

            if ((JCheckBox)source == mChBSimSuspendEnabled)
            {
                XPreferences.Write pw = XPreferences.getWriteInstance();
                pw.setSimSuspendEnabled(mChBSimSuspendEnabled.isSelected());
                storePreferences(pw);
                return;
            }
        }

        if (source instanceof JComboBox)
        {
            for (ESensorPort port : ESensorPort.values())
            {
                int i = port.ordinal();

                if ((JComboBox<String>)source == mCoBSensorType[i])
                {
                    XPreferences.Write pw = XPreferences.getWriteInstance();
                    ESensorType type = ESensorType.values()[mCoBSensorType[i].getSelectedIndex()];
                    ESensorDirection direction = BSensor.getDefaultSensorDirection(type);
                    mCoBDirection[i].removeActionListener(this);
                    initCoBDirections(i, type);
                    mCoBDirection[i].setSelectedItem(mSensorDirections[direction.ordinal()]);
                    mCoBDirection[i].addActionListener(this);
                    pw.setSensorType(port,type);
                    pw.setSensorDirection(port,direction);
                    storePreferences(pw);
                    return;
                }

                if ((JComboBox<String>)source == mCoBDirection[i])
                {
                    XPreferences.Write pw = XPreferences.getWriteInstance();
                    ESensorType type = mPrefs.getSensorType(port);
                    ESensorDirection direction = BSensor.getSensorDirections(type)[mCoBDirection[i].getSelectedIndex()];
                    pw.setSensorDirection(port,direction);
                    storePreferences(pw);
                    return;
                }
            }

            if ((JComboBox<XAudioSystem.CBMixerInfo>)source == mCoBMixerIn)
            {
                XPreferences.Write pw = XPreferences.getWriteInstance();
                XAudioSystem.CBMixerInfo cbmi = (XAudioSystem.CBMixerInfo) mCoBMixerIn.getSelectedItem();
                pw.setAudioMixerIn(cbmi.getKey());
                storePreferences(pw);
                return;
            }

            if ((JComboBox<XAudioSystem.CBMixerInfo>)source == mCoBMixerOut)
            {
                XPreferences.Write pw = XPreferences.getWriteInstance();
                XAudioSystem.CBMixerInfo cbmi = (XAudioSystem.CBMixerInfo) mCoBMixerOut.getSelectedItem();
                pw.setAudioMixerOut(cbmi.getKey());
                storePreferences(pw);
                return;
            }
        }

        if (source instanceof JButton)
        {
            if ((JButton)source == mBtnDefault)
            {
                XPreferences.reset();
                return;
            }

            if ((JButton)source == mBtnImport)
            {
                XPreferences.importFromXML();
                return;
            }

            if ((JButton)source == mBtnExport)
            {
                XPreferences.exportToXML();
                return;
            }
        }
    }

    private void storePreferences(XPreferences.Write prefs)
    {
        mIsSettingPreferences = true;
        prefs.store();
        mPrefs = prefs.getReadInstance();
        mIsSettingPreferences = false;
    }
    
    private float calculateDegrees(float orientation)
    {
        if (0f == orientation) return 0f;
        float degrees = orientation * 180f / D.PI_F;
        if (degrees < 0) degrees += 360f;
        return degrees;
    }

    /*static public void test()
    {
        //Locale locale = new Locale("en","US");
        Locale locale = new Locale("de","DE");
        XMsg.init(locale);
        
        JDialog dlg = new JDialog((JFrame)null,"Test",true);
        XSettingsPanel panel = new XSettingsPanel(null,true);
        dlg.add(panel);
        dlg.pack();
        panel.init(locale);
        dlg.setVisible(true);

    }*/
}
