/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 * 
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.test.core;

import org.jabotics.robot.core.*;
import org.jabotics.robot.defs.*;

import static org.jabotics.test.Assert.*;

import org.jabotics.robot.adapter.XAdapter;

/**
 * Beschreiben Sie hier die Klasse TestCore.
 * 
 * @author  Frank Gollnik
 * @version 1.1.2
 */
public class TestCore
{
    static private final IAdapterFactory sAdapterFactory = XAdapter.getFactory();
    static private final ISessionAdapter sSession = sAdapterFactory.getSessionAdapter();
    static
    {
        sSession.setSimulationMode(ESimulationMode.TEST);
    }
    static private ITimerAdapter sTimer;

    static public void setSimulationMode(ESimulationMode mode)
    {
        sSession.setSimulationMode(mode);
    }

    static public void prepare()
    {
        sSession.resetPreferences();
        if (null == sTimer)
        	sTimer = sAdapterFactory.getTimerAdapter();
        else
            sSession.reset();
    }

    static public void cleanup()
    {
    }

     static public void main(String[] args)
    {
    	//testMotor3();
    	testAll();
    }

    static public void testAll()
    {
        testMotor0();
        testMotor1();
        testMotor2();
        testMotor3();
        testMotor4();
        testMotor5();
        testMotor6();
        testMotor7();
        testMotor8();
        testMotor9();
        testPilot1();
        testPilot2();
        testPilot3();
        testPilot4();
    }

    static public void testMotor0()
    {
        prepare();
        execTestMotor0();
        cleanup();
    }

    static private class TestMotor0Thread extends Thread
    {
        final CMotor.XTransaction mTransaction;
        long mCount = 0;
        TestMotor0Thread(CMotor.XTransaction transaction) { super(); mTransaction = transaction; }
        @Override public void run()
        {
            while (true)
            {
                if (this.isInterrupted()) return;
                // pause between transactions:
                for (int i = 0, s = 1000 + (int)(3000*Math.random()); i < s; ++i);
                
                mTransaction.start();
                ++mCount;
                // pause within transaction:
                for (int i = 0, s = 100 + (int)(1000*Math.random()); i < s; ++i);
                mTransaction.commit(false);
            }
        }
    }

    static private void execTestMotor0()
    {
        CMotor motor1 = CMotor.getMotor(EMotorPort.A);
        CMotor motor2 = CMotor.getMotor(EMotorPort.B);
        CMotor motor3 = CMotor.getMotor(EMotorPort.C);

        CMotor.XTransaction transaction1 = new CMotor.XTransaction(motor1);
        CMotor.XTransaction transaction2 = new CMotor.XTransaction(motor2);
        CMotor.XTransaction transaction3 = new CMotor.XTransaction(motor3);
        CMotor.XTransaction transaction12 = new CMotor.XTransaction(motor1,motor2);
        CMotor.XTransaction transaction13 = new CMotor.XTransaction(motor1,motor3);
        CMotor.XTransaction transaction23 = new CMotor.XTransaction(motor2,motor3);
        CMotor.XTransaction transaction123 = new CMotor.XTransaction(motor1,motor2,motor3);

        TestMotor0Thread thread1 = new TestMotor0Thread(transaction1);
        TestMotor0Thread thread2 = new TestMotor0Thread(transaction2);
        TestMotor0Thread thread3 = new TestMotor0Thread(transaction3);
        TestMotor0Thread thread12 = new TestMotor0Thread(transaction12);
        TestMotor0Thread thread13 = new TestMotor0Thread(transaction13);
        TestMotor0Thread thread23 = new TestMotor0Thread(transaction23);
        TestMotor0Thread thread123 = new TestMotor0Thread(transaction123);

        long start = sTimer.currentTimeMillis();

        thread1.start();
        thread2.start();
        thread3.start();
        thread12.start();
        thread13.start();
        thread23.start();
        thread123.start();
        sTimer.msDelay(10000);
        thread1.interrupt();
        thread2.interrupt();
        thread3.interrupt();
        thread12.interrupt();
        thread13.interrupt();
        thread23.interrupt();
        thread123.interrupt();
        sTimer.msDelay(2000);

        long duration = sTimer.currentTimeMillis() - start;

        long count1 = thread1.mCount;
        long count2 = thread2.mCount;
        long count3 = thread3.mCount;
        long count12 = thread12.mCount;
        long count13 = thread13.mCount;
        long count23 = thread23.mCount;
        long count123 = thread123.mCount;

        assertEquals("Execution time (" + duration + "ms) does not meet expectation of 12000ms.", 12000, duration, 50);
        assert !thread1.isAlive() : "Thread 1 is still alive.";
        assert !thread2.isAlive() : "Thread 2 is still alive.";
        assert !thread3.isAlive() : "Thread 3 is still alive.";
        assert !thread12.isAlive() : "Thread 12 is still alive.";
        assert !thread13.isAlive() : "Thread 13 is still alive.";
        assert !thread23.isAlive() : "Thread 23 is still alive.";
        assert !thread123.isAlive() : "Thread 123 is still alive.";
        assert count1 > 1500000 : "Iteration count of thread 1 does not meet minimum expectation for reference PC (is " + count1 + ").";
        assert count2 > 1500000 : "Iteration count of thread 2 does not meet minimum expectation for reference PC (is " + count2 + ").";
        assert count3 > 1500000 : "Iteration count of thread 3 does not meet minimum expectation for reference PC (is " + count3 + ").";
        assert count12 > 500000 : "Iteration count of thread 12 does not meet minimum expectation for reference PC (is " + count12 + ").";
        assert count13 > 500000 : "Iteration count of thread 13 does not meet minimum expectation for reference PC (is " + count13 + ").";
        assert count23 > 500000 : "Iteration count of thread 23 does not meet minimum expectation for reference PC (is " + count23 + ").";
        assert count123 > 150000 : "Iteration count of thread 123 does not meet minimum expectation for reference PC (is " + count123 + ").";
    }

    static public void testMotor1()
    {
        prepare();
        execTestMotor1();
        cleanup();
    }

    static private void execTestMotor1()
    {
        CMotor motor1 = CMotor.getMotor(EMotorPort.B);
        CMotor motor2 = CMotor.getMotor(EMotorPort.C);
        CMotor.XTransaction transaction1 = new CMotor.XTransaction(motor1);
        CMotor.XTransaction transaction2 = new CMotor.XTransaction(motor2);

        // motor settings to default values:
        transaction1.start();
        transaction2.start();
        motor1.setTargetSpeed(transaction1,360);
        motor1.setTargetAcceleration(transaction1,6000);
        motor2.setTargetSpeed(transaction2,360);
        motor2.setTargetAcceleration(transaction2,6000);
        transaction1.commit(false);
        transaction2.commit(false);

        sTimer.msDelay(1000);

        transaction1.start();
        motor1.startRotationUnlimited(transaction1,true);
        transaction1.commit(false);

        sTimer.msDelay(4000);

        transaction2.start();
        motor2.startRotationTimeLimited(transaction2,8000f,true);
        transaction2.commit(false);

        sTimer.msDelay(4000);

        transaction1.start();
        motor1.startRotationUnlimited(transaction1,false);
        transaction1.commit(false);

        sTimer.msDelay(10000);

        transaction1.start();
        motor1.stopRotation(transaction1);
        transaction1.commit(false);

        sTimer.msDelay(2000);

        assertEquals("Position motor 1 does not meet expectation.", -698f, motor1.getPosition(), 5f);
        assertEquals("Position motor 2 does not meet expectation.", 2858f, motor2.getPosition(), 5f);
    }

    static public void testMotor2()
    {
        prepare();
        execTestMotor2();
        cleanup();
    }

    static private void execTestMotor2()
    {
        CMotor motor1 = CMotor.getMotor(EMotorPort.B);
        CMotor motor2 = CMotor.getMotor(EMotorPort.C);
        CMotor.XTransaction transaction1 = new CMotor.XTransaction(motor1);
        CMotor.XTransaction transaction2 = new CMotor.XTransaction(motor2);

        sTimer.msDelay(1000);

        transaction1.start();
        motor1.setTargetSpeed(transaction1,600);
        motor1.setTargetAcceleration(transaction1,300);
        motor1.startRotationUnlimited(transaction1,true);
        transaction1.commit(false);

        sTimer.msDelay(4000);

        transaction2.start();
        motor2.setTargetSpeed(transaction2,300);
        motor2.setTargetAcceleration(transaction2,300);
        motor2.startRotationTimeLimited(transaction2,8000f,true);
        transaction2.commit(false);

        sTimer.msDelay(4000);

        transaction1.start();
        motor1.setTargetSpeed(transaction1,300);
        motor1.startRotationTimeLimited(transaction1,8000f,false);
        transaction1.commit(false);

        sTimer.msDelay(12000);

        assertEquals("Position motor 1 does not meet expectation (is: " + motor1.getPosition() + ").", 3300f, motor1.getPosition(), 5f);
        assertEquals("Position motor 2 does not meet expectation (is: " + motor2.getPosition() + ").", 2100f, motor2.getPosition(), 5f);
    }

    static public void testMotor3()
    {
        prepare();
        execTestMotor3();
        cleanup();
    }

    static private void execTestMotor3()
    {
        CMotor motor1 = CMotor.getMotor(EMotorPort.B);
        CMotor.XTransaction transaction1 = new CMotor.XTransaction(motor1);

        sTimer.msDelay(4000);

        transaction1.start();
        motor1.setTargetSpeed(transaction1,600);
        motor1.setTargetAcceleration(transaction1,150);
        motor1.startRotationUnlimited(transaction1,true);
        transaction1.commit(false);

        sTimer.msDelay(6000);

        transaction1.start();
        motor1.startRotationTimeLimited(transaction1,2000f,false);
        transaction1.commit(false);

        sTimer.msDelay(3000);

        assertEquals("Position motor 1 does not meet expectation.", 3528f, motor1.getPosition(), 5f);
    }

    static public void testMotor4()
    {
        prepare();
        execTestMotor4();
        cleanup();
    }

    static private void execTestMotor4()
    {
        CMotor motor1 = CMotor.getMotor(EMotorPort.B);
        CMotor.XTransaction transaction1 = new CMotor.XTransaction(motor1);

        sTimer.msDelay(4000);

        transaction1.start();
        motor1.setTargetSpeed(transaction1,600);
        motor1.setTargetAcceleration(transaction1,300);
        motor1.startRotationPosLimited(transaction1,7200,true);
        transaction1.commit(true);

        sTimer.msDelay(4000);

        transaction1.start();
        motor1.startRotationPosLimited(transaction1,3600,false);
        transaction1.commit(true);

        sTimer.msDelay(4000);

        transaction1.start();
        motor1.resetPosition();
        motor1.startRotationPosLimited(transaction1,3600,false);
        transaction1.commit(true);

        sTimer.msDelay(4000);

        assertEquals("Position motor 1 does not meet expectation.", 3600f, motor1.getPosition(), 5f);
    }

    static public void testMotor5()
    {
        prepare();
        execTestMotor5();
        cleanup();
    }

    static private void execTestMotor5()
    {
        CMotor motor1 = CMotor.getMotor(EMotorPort.B);
        CMotor.XTransaction transaction1 = new CMotor.XTransaction(motor1);

        sTimer.msDelay(4000);

        transaction1.start();
        motor1.setTargetSpeed(transaction1,400);
        motor1.setTargetAcceleration(transaction1,200);
        motor1.startRotationUnlimited(transaction1,false);
        transaction1.commit(false);

        sTimer.msDelay(6000);

        transaction1.start();
        motor1.setTargetSpeed(transaction1,800);
        motor1.startRotationPosLimited(transaction1,-800,true);
        transaction1.commit(true);

        sTimer.msDelay(4000);

        assertEquals("Position motor 1 does not meet expectation.", -2800f, motor1.getPosition(), 5f);
    }

    static public void testMotor6()
    {
        prepare();
        execTestMotor6();
        cleanup();
    }

    static private void execTestMotor6()
    {
        CMotor motor1 = CMotor.getMotor(EMotorPort.B);
        CMotor motor2 = CMotor.getMotor(EMotorPort.C);
        CMotor.XTransaction transaction = new CMotor.XTransaction(motor1,motor2);

        sTimer.msDelay(4000);

        transaction.start();
        motor1.setTargetSpeed(transaction,300);
        motor1.setTargetAcceleration(transaction,200);
        motor1.startRotationUnlimited(transaction,true);
        motor2.setTargetSpeed(transaction,300);
        motor2.setTargetAcceleration(transaction,200);
        motor2.startRotationUnlimited(transaction,true);
        transaction.commit(false);

        sTimer.msDelay(4000);

        transaction.start();
        motor1.setTargetSpeed(transaction,400);
        motor1.setTargetAcceleration(transaction,200);
        motor2.setTargetSpeed(transaction,200);
        motor2.setTargetAcceleration(transaction,100);
        motor1.startRotationPosLimited(transaction,2000,400,0.5f);
        motor2.startRotationPosLimited(transaction,1000,200,0.5f);
        transaction.commit(true);

        sTimer.msDelay(4000);

        assertEquals("Position motor 1 does not meet expectation.", 2974.4f, motor1.getPosition(), 5f);
        assertEquals("Position motor 2 does not meet expectation.", 1974.4f, motor2.getPosition(), 5f);
    }

    static public void testMotor7()
    {
        prepare();
        execTestMotor7();
        cleanup();
    }

    static private void execTestMotor7()
    {
        CMotor motor1 = CMotor.getMotor(EMotorPort.B);
        CMotor motor2 = CMotor.getMotor(EMotorPort.C);
        CMotor.XTransaction transaction = new CMotor.XTransaction(motor1,motor2);

        sTimer.msDelay(4000);

        transaction.start();
        motor1.setTargetSpeed(transaction,400);
        motor1.setTargetAcceleration(transaction,200);
        motor1.startRotationUnlimited(transaction,true);
        motor2.setTargetSpeed(transaction,400);
        motor2.setTargetAcceleration(transaction,200);
        motor2.startRotationUnlimited(transaction,true);
        transaction.commit(false);

        sTimer.msDelay(4000);

        transaction.start();
        motor1.setTargetSpeed(transaction,400);
        motor1.setTargetAcceleration(transaction,200);
        motor2.setTargetSpeed(transaction,400);
        motor2.setTargetAcceleration(transaction,200);
        motor1.startRotationPosLimited(transaction,2000,0,2f);
        motor2.startRotationPosLimited(transaction,-2000,0,2f);
        transaction.commit(true);

        sTimer.msDelay(4000);

        assertEquals("Position motor 1 does not meet expectation.", 3200f, motor1.getPosition(), 5f);
        assertEquals("Position motor 2 does not meet expectation.", -800f, motor2.getPosition(), 5f);
    }

    static public void testMotor8()
    {
        prepare();
        execTestMotor8();
        cleanup();
    }

    static private void execTestMotor8()
    {
        CMotor motor1 = CMotor.getMotor(EMotorPort.B);
        CMotor motor2 = CMotor.getMotor(EMotorPort.C);
        CMotor.XTransaction transaction = new CMotor.XTransaction(motor1,motor2);

        sTimer.msDelay(4000);

        transaction.start();
        motor1.setTargetSpeed(transaction,400);
        motor1.setTargetAcceleration(transaction,200);
        motor1.startRotationUnlimited(transaction,true);
        motor2.setTargetSpeed(transaction,400);
        motor2.setTargetAcceleration(transaction,200);
        motor2.startRotationUnlimited(transaction,true);
        transaction.commit(false);

        sTimer.msDelay(4000);

        transaction.start();
        motor1.setTargetSpeed(transaction,400);
        motor1.setTargetAcceleration(transaction,200);
        motor2.setTargetSpeed(transaction,200);
        motor2.setTargetAcceleration(transaction,100);
        motor1.startRotationPosLimited(transaction,2000,400,2f);
        motor2.startRotationPosLimited(transaction,-1000,-200,2f);
        transaction.commit(true);

        sTimer.msDelay(4000);

        assertEquals("Position motor 1 does not meet expectation.", 3202f, motor1.getPosition(), 5f);
        assertEquals("Position motor 2 does not meet expectation.", 202f, motor2.getPosition(), 5f);
    }

    static public void testMotor9()
    {
        prepare();
        execTestMotor9();
        cleanup();
    }

    static private void execTestMotor9()
    {
        // Test items:
        // (a) changing motor speed and acceleration during rotation
        // (b) method CMotor.waitUntilCompleted
        // (c) method CMotor.isMoving
        CMotor motor = CMotor.getMotor(EMotorPort.B);
        CMotor.XTransaction transaction = new CMotor.XTransaction(motor);
        //long start = sTimer.currentTimeMillis();

        // First phase
        // change speed and acceleration immediately after starting the motion
        // (probably before controller will handle new motion)
        sTimer.msDelay(1000);
        transaction.start();
        motor.setTargetSpeed(transaction,400);
        transaction.commit(false);
        transaction.start();
        motor.setTargetAcceleration(transaction,200);
        transaction.commit(false);
        transaction.start();
        motor.startRotationUnlimited(transaction,true);
        transaction.commit(false);
        transaction.start();
        motor.setTargetSpeed(transaction,200);
        motor.setTargetAcceleration(transaction,100);
        transaction.commit(false);
        sTimer.msDelay(4000);
        transaction.start();
        motor.startRotationTimeLimited(transaction,0f,true); // halt motion
        transaction.commit(false);
        transaction.waitUntilMotionCompleted();
        float pos1 = motor.getPosition();

        // Second phase
        // change speed and acceleration shortly after starting the motion
        // (probably after controller has initialzed new motion)
        sTimer.msDelay(1000);
        transaction.start();
        motor.setTargetSpeed(transaction,400);
        motor.setTargetAcceleration(transaction,200);
        motor.startRotationUnlimited(transaction,false);
        transaction.commit(false);
        sTimer.msDelay(20);
        transaction.start();
        motor.setTargetSpeed(transaction,200);
        motor.setTargetAcceleration(transaction,100);
        transaction.commit(false);
        transaction.start();
        sTimer.msDelay(3980);
        motor.startRotationTimeLimited(transaction,0f,true); // halt motion
        transaction.commit(false);
        transaction.waitUntilMotionCompleted();
        float pos2 = motor.getPosition();

        // Third phase
        // change speed and acceleration "in the middle" of the motion
        sTimer.msDelay(1000);
        transaction.start();
        motor.setTargetSpeed(transaction,400);
        motor.setTargetAcceleration(transaction,200);
        motor.startRotationUnlimited(transaction,true);
        transaction.commit(false);
        sTimer.msDelay(4000);
        transaction.start();
        motor.setTargetSpeed(transaction,200);
        motor.setTargetAcceleration(transaction,100);
        transaction.commit(false);
        sTimer.msDelay(4000);
        transaction.start();
        motor.startRotationTimeLimited(transaction,0f,true); // halt motion
        transaction.commit(false);
        while (motor.isMoving()) sTimer.msDelay(1);
        float pos3 = motor.getPosition();

        // Fourth phase
        // change speed and acceleration "in the middle" of the motion
        // which is now time limitted
        sTimer.msDelay(1000);
        transaction.start();
        motor.setTargetSpeed(transaction,400);
        motor.setTargetAcceleration(transaction,200);
        motor.startRotationTimeLimited(transaction,8000,false);
        transaction.commit(false);
        sTimer.msDelay(4000);
        transaction.start();
        motor.setTargetSpeed(transaction,200);
        motor.setTargetAcceleration(transaction,200);
        transaction.commit(false);
        transaction.waitUntilMotionCompleted();
        float pos4 = motor.getPosition();

        // Fifth phase
        // change speed and acceleration "in the middle" of the motion
        // which is now limitted by a relative position
        sTimer.msDelay(1000);
        long t0 = sTimer.currentTimeMillis();
        transaction.start();
        motor.setTargetSpeed(transaction,400);
        motor.setTargetAcceleration(transaction,200);
        motor.startRotationPosLimited(transaction,2200,true);
        transaction.commit(false);
        sTimer.msDelay(4000);
        transaction.start();
        motor.setTargetSpeed(transaction,200);
        transaction.commit(false);
        transaction.waitUntilMotionCompleted();
        long t5 = sTimer.currentTimeMillis() - t0;

        // Sixth phase
        // change speed and acceleration "in the middle" of the motion
        // which is now limitted by an absolute position
        sTimer.msDelay(1000);
        t0 = sTimer.currentTimeMillis();
        transaction.start();
        float s0 = motor.getPosition();
        float s1 = s0 + 1200;
        float s2 = s0 + 2200;
        motor.setTargetSpeed(transaction,400);
        motor.setTargetAcceleration(transaction,200);
        motor.startRotationPosLimited(transaction,s2,false);
        transaction.commit(false);
        while (motor.getPosition() < s1) sTimer.msDelay(1);
        transaction.start();
        motor.setTargetSpeed(transaction,200);
        motor.setTargetAcceleration(transaction,100);
        transaction.commit(false);
        transaction.waitUntilMotionCompleted();
        long t6 = sTimer.currentTimeMillis() - t0;

        // Seventh phase
        // reduce acceleration "in the middle" of the motion
        // (limitted by a relative position) which is now
        // resulting in a final backward movement
        sTimer.msDelay(1000);
        t0 = sTimer.currentTimeMillis();
        transaction.start();
        motor.setTargetSpeed(transaction,400);
        motor.setTargetAcceleration(transaction,200);
        motor.startRotationPosLimited(transaction,2200,true);
        transaction.commit(false);
        sTimer.msDelay(5000);
        transaction.start();
        motor.setTargetAcceleration(transaction,100);
        transaction.commit(false);
        transaction.waitUntilMotionCompleted();
        long t7 = sTimer.currentTimeMillis() - t0;

        //long duration = sTimer.currentTimeMillis() - start;

        assertEquals("First position of motor does not meet expectation.",  800f, pos1,  5f);
        assertEquals("Second position of motor does not meet expectation.",   0f, pos2, 10f);
        assertEquals("Third position of motor does not meet expectation.", 2400f, pos3, 15f);
        assertEquals("Fourth position of motor does not meet expectation.", 400f, pos4, 20f);
        assertEquals("Fifth execution time of motor does not meet expectation.", 9000f, t5, 20f);
        assertEquals("Sixth execution time of motor does not meet expectation.", 9000f, t6, 20f);
        assertEquals("Seventh execution time of motor does not meet expectation.", 11828f, t7, 20f);
    }

    static public void testPilot1()
    {
        prepare();
        execTestPilot1();
        cleanup();
    }

    static private void execTestPilot1()
    {
        CMotor motor1 = CMotor.getMotor(EMotorPort.B);
        CMotor motor2 = CMotor.getMotor(EMotorPort.C);
        CPilot pilot = CPilot.getPilotSingleton(EMotorPort.B,EMotorPort.C);
        sTimer.msDelay(1000);
        pilot.setTargetSpeed(20);
        pilot.setTargetAcceleration(10);
        pilot.moveStraight(true);
        sTimer.msDelay(6000);
        pilot.moveCurvature(50f,true);
        sTimer.msDelay(6000);
        pilot.stopMotion(true);
        sTimer.msDelay(1000);

        float pos1 = motor1.getPosition();
        float pos2 = motor2.getPosition();

        assertEquals("Position motor 1 does not meet expectation.", 2454.3f, pos1, 5f);
        assertEquals("Position motor 2 does not meet expectation.", 4910.6f, pos2, 5f);
    }

    static public void testPilot2()
    {
        prepare();
        execTestPilot2();
        cleanup();
    }

    static private void execTestPilot2()
    {
        CMotor motor1 = CMotor.getMotor(EMotorPort.B);
        CMotor motor2 = CMotor.getMotor(EMotorPort.C);
        CPilot pilot = CPilot.getPilotSingleton(EMotorPort.B,EMotorPort.C);
        sTimer.msDelay(4000);
        pilot.setTargetSpeed(20);
        pilot.setTargetAcceleration(10);
        pilot.moveStraight(true);
        sTimer.msDelay(6000);
        pilot.moveArc(50f,-30,true);
        //sTimer.msDelay(6000);
        pilot.stopMotion(true);
        sTimer.msDelay(2000);

        assertEquals("Position motor 1 does not meet expectation.", 3130f, motor1.getPosition(), 5f);
        assertEquals("Position motor 2 does not meet expectation.", 3010f, motor2.getPosition(), 5f);
    }

    static public void testPilot3()
    {
        prepare();
        execTestPilot3();
        cleanup();
    }

    static private void execTestPilot3()
    {
        CMotor motor1 = CMotor.getMotor(EMotorPort.B);
        CMotor motor2 = CMotor.getMotor(EMotorPort.C);
        CPilot pilot = CPilot.getPilotSingleton(EMotorPort.B,EMotorPort.C);
        sTimer.msDelay(1000);
        pilot.setTargetSpeed(20);
        pilot.setTargetAcceleration(10);

        float end = sTimer.currentTimeMillis() + 6000f;

        while (end > sTimer.currentTimeMillis())
        {
            // stupid, but this is what students will do
            pilot.moveStraight(true);
        }
        while (pilot.getOrientation() < 60f)
        {
            // stupid, but this is what students will do
            pilot.moveRadius(100f,true);
        }
        pilot.stopMotion(true);
        sTimer.msDelay(2000);
        
        assertEquals("Position motor 1 does not meet expectation.", 4479f, motor1.getPosition(), 5f);
        assertEquals("Position motor 2 does not meet expectation.", 4763f, motor2.getPosition(), 5f);
    }

    static public void testPilot4()
    {
        prepare();
        execTestPilot4();
        cleanup();
    }

    static private void execTestPilot4()
    {
        // Test items:
        // (a) changing speed and acceleration during motion
        // (b) method CPilot.waitUntilCompleted
        // (c) method CPilot.isMoving
        CPilot pilot = CPilot.getPilotSingleton(EMotorPort.B,EMotorPort.C);
        long start = sTimer.currentTimeMillis();

        sTimer.msDelay(2000);
        pilot.setTargetSpeed(10);
        pilot.setTargetAcceleration(10);
        pilot.moveStraight(100,false);
        sTimer.msDelay(4000);
        pilot.setTargetSpeed(20);
        sTimer.msDelay(2000);
        pilot.setTargetAcceleration(20);
        pilot.waitUntilCompleted();

        float t1 = sTimer.currentTimeMillis() - start;
        float pos1 = pilot.getTachoCount();
        float dir1 = pilot.getOrientation();

        sTimer.msDelay(2000);
        pilot.setTargetSpeed(20);
        pilot.setTargetAcceleration(20);
        pilot.moveStraight(false);
        sTimer.msDelay(3000);
        pilot.setTargetAcceleration(1);
        pilot.moveArc(-100,180,false);
        sTimer.msDelay(2000);
        pilot.setTargetAcceleration(20);
        while (pilot.isMoving()) sTimer.msDelay(1);

        float t2 = sTimer.currentTimeMillis() - start - t1;
        float pos2 = pilot.getTachoCount();
        float dir2 = pilot.getOrientation();

        sTimer.msDelay(2000);
        pilot.setTargetSpeed(1);
        pilot.setTargetAcceleration(100);
        pilot.moveCurvature(100,false);
        for (int i = 0; i <20; ++i)
        {
            sTimer.msDelay(500);
            pilot.setTargetSpeed(i);
        }
        for (int i = 20; i > 0; --i)
        {
            sTimer.msDelay(500);
            pilot.setTargetSpeed(i);
        }
        pilot.stopMotion(true);

        float t3 =  sTimer.currentTimeMillis() - start - t1 - t2;
        float pos3 = pilot.getTachoCount();
        float dir3 = pilot.getOrientation();

        sTimer.msDelay(2000);
        
        assertEquals("First execution time does not meet expectation.", 10005f, t1, 10f);
        assertEquals("First position does not meet expectation.", 100f, pos1, 0.2f);
        assertEquals("First orientation does not meet expectation.", 0f, dir1, 0.2f);
        assertEquals("Second execution time does not meet expectation.", 11339f, t2, 10f);
        assertEquals("Second position does not meet expectation.", -50f, pos2, 0.2f);
        assertEquals("Second orientation does not meet expectation.", 180f, dir2, 0.2f);
        assertEquals("Third execution time does not meet expectation.", 22024f, t3, 10f);
        assertEquals("Third position does not meet expectation.", -50f, pos3, 0.2f);
        assertEquals("Third orientation does not meet expectation.", 2232f, dir3, 1f);
    }
}
