/********************************************************************************************
*
* Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
*
* 
* This file is part of the Java software package org.jabotics (in the following: Jabotics).
* 
* Jabotics is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
* 
* Jabotics is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
* 
* 
* Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
* 
* Jabotics ist Freie Software: Sie können es unter den Bedingungen
* der GNU General Public License, wie von der Free Software Foundation,
* Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
* veröffentlichten Version, weiterverbreiten und/oder modifizieren.
* 
* Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
* OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
* Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
* Siehe die GNU General Public License für weitere Details.
* 
* Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
* erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
* 
*******************************************************************************************/


package org.jabotics.test.ipc;

import static org.junit.Assert.fail;

import java.io.File;
import java.rmi.Naming;
import java.rmi.RemoteException;
import java.util.ArrayList;
//import java.util.Calendar;

public class XTestRMIClient
{
    static final long TEST_PERIOD = 10000;
    static final boolean WINDOWS = ('\\' == File.separatorChar);
    static int i = 0;
    static int j = 0;
    static int errors = 0;
    static int[] field = null;

    static public void main(String[] args)
    {
        // working directory (java property "user.dir") is project directory in Eclipse
        String classpath = System.getProperty("user.dir") + File.separatorChar + "bin";
        String java = System.getProperty("java.home") + (WINDOWS ? "\\bin\\java.exe" : "/bin/java");

        ITestRMIServer server = null; 

        try
        {
            server = (ITestRMIServer)Naming.lookup(XTestRMIServer.SERVER_NAME);
        }
        catch (Exception e)  {}

        try
        {
            if (null == server)
            {
                // Server has to be started
                ArrayList<String> commandList = new ArrayList<String>();
                ProcessBuilder pb = new ProcessBuilder(commandList);
                commandList.add(java);
                commandList.add("-cp");
                commandList.add(classpath);
                commandList.add("org.jabotics.test.ipc.XTestRMIServer");
                pb.start();
                pause(500);
                server = (ITestRMIServer)Naming.lookup(XTestRMIServer.SERVER_NAME);
            }

            final ITestRMIWorker worker = server.getWorker();
            final long end = System.currentTimeMillis() + TEST_PERIOD;
            (new Thread() { public void run() {
                try
                {
                    while (System.currentTimeMillis() < end)
                    {
                        i = worker.getCount();
                    }
                }
                catch (RemoteException e)
                {
                    e.printStackTrace();
                }
            }}).start();
            while (System.currentTimeMillis() < end)
            {
                ++j;
                field = worker.getField();
                if (field[0]!=XTestRMIWorker.FIELD[0]||field[1]!=XTestRMIWorker.FIELD[1]||field[2]!=XTestRMIWorker.FIELD[2]||field[3]!=XTestRMIWorker.FIELD[3]) ++errors;
            }
            pause(500);
            System.out.println("Count(i): " + i);
            System.out.println("Count(j): " + j);
            System.out.println("Errors(j): " + errors);
            System.out.print("Values: ");
            for (int value : field) System.out.print(value + " ; ");
            System.out.println("Number of elements: " + field.length);
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }

        try
        {
            // Clean up server including rmi registry:
            server.terminate();
            // Check for success in terminal with: jps -l 
        }
        catch (Exception e)  {}

    
    
        ITestRMISingleton singleton_server = null; 
        ITestRMISingleWorker singleton_worker = null;


        for (int i = 0;  i < 14;  ++i)
        {
            try
            {
                singleton_server = (ITestRMISingleton)Naming.lookup(XTestRMISingleton.SINGLETON_NAME);
            }
            catch (Exception e)  {}

            if (null == singleton_server)
            {
                try
                {
                    System.out.println("Singleton server has to be started.");

                    ArrayList<String> commandList = new ArrayList<String>();
                    ProcessBuilder pb = new ProcessBuilder(commandList);
                    commandList.add(java);
                    commandList.add("-cp");
                    commandList.add(classpath);
                    commandList.add("org.jabotics.test.ipc.XTestRMISingleton");
                    pb.start();
                    pause(500);
                    singleton_server = (ITestRMISingleton)Naming.lookup(XTestRMISingleton.SINGLETON_NAME);
                }
                catch (Exception e)  {}
            }

            try
            {
                singleton_worker = singleton_server.getSingleWorker();
            }
            catch (Exception e)  {}

            if (null != singleton_worker)
            {
                System.out.println("Succeeded to load new singleton: Will be busy for a while.");
                try
                {
                    singleton_worker.terminate();
                }
                catch (Exception e)  {}
            }
            else
            {
                System.out.println("Failed to load singleton: Worker singleton still busy.");
            }

            // Give rmi registry a chance to clean up singleton server:
            singleton_server = null; 
            singleton_worker = null;
            pause(1000);
        }
    }

    private static void pause(long period_ms)
    {
        try
        {
            Thread.sleep(period_ms);
        }
        catch (InterruptedException e)
        {
            fail("Thread has been interrupted.");
        }
    }
}
