/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 * 
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/

package org.jabotics.test.robot;


import static org.jabotics.test.Assert.*;

import org.jabotics.robot.de.*;

/**
 * Beschreibe hier die Klasse TestToTheLight.
 * 
 * @author  Frank Gollnik
 * @version 1.1.2
 */
public class TestToTheLight extends BTestRoboter implements ISensorbeobachter
{
    static final float MINDESTABSTAND = 40f;
    static final float KORREKTURWINKEL = 20f;
    static final float RUECKSETZWINKEL = 50f;
    static final long TIMEOUT = 60000; // in ms

    float hMax; // maximale Helligkeit
    float wMax; // zugehöriger Ausrichtungswinkel;
    float mGesamtstrecke = 0f;
    float mGesamtdrehwinkel = 0f;
    long mStartzeitpunkt;
    long mEndzeitpunkt;
    

    public static void main(String[] args)
    {
        TestToTheLight roboter = new TestToTheLight();
        roboter.run();
    }

    /**
     * Hier können bei Bedarf zusätzlich Initialisierungen einer mit dem new-Operator
     * erzeugten Roboterinstanz vorgenommen werden.
     */
    public TestToTheLight()
    {
    }

    @Override
    public void run()
    {
        mStartzeitpunkt = uhr.holeSystemzeit();

        uhr.warteZeit(2000);

        //float richtung = 1f;
        float korrekturrichtung = 1f;
        //float korrekturwinkel = 0f;
        int i = 0;
        do
        {
            setzeZaehlerstaendeZurueck();
            pilot.setzeSollgeschwindigkeit(5);
            pilot.setzeSollbeschleunigung(10);
            hMax = lichtsensor.holeMesswert();              // Startwert für die größte bisher gefundenen Helligkeit
            wMax = pilot.holeAusrichtungswinkel();          // Startwert für den zugehörigen Ausrichtungswinkel (=0)
            lichtsensor.setzeBeobachter(this);              // das Roboterobjekt wird als Beobachter angemeldet
            pilot.dreheZuAusrichtungswinkel(360,ABWARTEN);  // Roboter dreht sich einmal um 360°
                                                            // Währenddessen meldet der Sensor Änderungsereignisse
            lichtsensor.loescheBeobachter();                // Überwachung des Sensors wird beendet
            if (wMax < 180f) wMax = wMax + 360f;            // eine kleine Optimierung für folgenden Schritt
            pilot.dreheZuAusrichtungswinkel(wMax,ABWARTEN); // Hindrehen zur Richtung mit der größten registrierten Helligkeit.

            setzeZaehlerstaendeZurueck();
            float ausweichwinkel = 0f;
            float s = ultraschallsensor.holeMesswert();
            pilot.dreheZuAusrichtungswinkel(-10f,ABWARTEN);
            ausweichwinkel += 0.1f * (s - ultraschallsensor.holeMesswert());
            pilot.dreheZuAusrichtungswinkel(10f,ABWARTEN);
            ausweichwinkel -= 0.1f * (s - ultraschallsensor.holeMesswert());
            pilot.dreheZuAusrichtungswinkel(ausweichwinkel,ABWARTEN);

            if (ultraschallsensor.holeMesswert() > MINDESTABSTAND)
            {
                pilot.setzeSollbeschleunigung(30);
                pilot.setzeSollgeschwindigkeit(15);
                pilot.fahreGerade(VORWAERTS);         // Roboter bleibt rechtzeitig vor Hindernis stehen.
                long ende = uhr.holeSystemzeit() + TIMEOUT;
                int stalled_counter = 0;
                do
                {
                    uhr.warteZeit(200);

                    s = ultraschallsensor.holeMesswert();
                    float v = ultraschallsensor.holeAenderungsrate();
                    if ((s < 220f) && (v > -0.1f) && (pilot.holeGeschwindigkeit() > 10f))
                    {
                        ++stalled_counter;
                    }
                    else
                    {
                        stalled_counter = 0;
                    }

                    if ((uhr.holeSystemzeit() > ende) || (5 < stalled_counter))
                    {
                        // Kollision
                        pilot.fahreBogen(-MINDESTABSTAND,korrekturrichtung*RUECKSETZWINKEL,ABWARTEN);
                        korrekturrichtung = -korrekturrichtung;
                        //korrekturwinkel = korrekturrichtung * KORREKTURWINKEL;
                        break;
                    }
                    else if (s < MINDESTABSTAND)
                    {
                        // Hindernis
                        pilot.halteAn(ABWARTEN);
                        //korrekturwinkel = 0f;
                        break;
                    }
                } while (true);
            }
            
            i = (pilot.holeTachostand() > 30) ? 0 : i + 1;
        }
        while (i<5);

        mEndzeitpunkt = uhr.holeSystemzeit();
    }

    @Override
    public void registriereEreignis(ISensor sensor, float alterWert, float neuerWert, float alteRate, float neueRate)
    {
        if (lichtsensor.equals(sensor))
        {
            if (neuerWert > hMax)
            {
                hMax = neuerWert;
                wMax = pilot.holeAusrichtungswinkel();
            }
            return;
        }
    }

    private void setzeZaehlerstaendeZurueck()
    {
        mGesamtstrecke += pilot.holeTachostand();
        mGesamtdrehwinkel += pilot.holeAusrichtungswinkel();
        pilot.setzeZaehlerstaendeZurueck();
    }

    public void trailer()
    {
        uhr.warteZeit(2000);

        boolean invertiert = false;
        for (int i = 0;  i < 8;  ++i)
        {
            invertiert = !invertiert;
            monitor.schreibe("Ans Licht!", 3, 2, invertiert);
            monitor.schreibe("To the light!!", 1, 4, invertiert);
            uhr.warteZeit(500);
        }
    }

    @Override
    public void checkAsserts()
    {
        /*System.out.println("TestToTheLight:");
        System.out.println("Start:   " + mStartzeitpunkt);
        System.out.println("Ende:    " + mEndzeitpunkt);
        System.out.println("Dauer:   " + (mEndzeitpunkt - mStartzeitpunkt));
        System.out.println("Strecke: " + mGesamtstrecke);
        System.out.println("Winkel:  " + mGesamtdrehwinkel);*/

        assertEquals("Total distance does not meet expection.", 2200f, mGesamtstrecke, 100f);
        assertEquals("Total robot rotation does not meet expection.", 4400f, mGesamtdrehwinkel, 500f);
        assertEquals("Execution time does not meet expection.", 278000, (mEndzeitpunkt - mStartzeitpunkt), 15000);
    }
}
