/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.tools; 

import bluej.extensions.*;
import java.net.URL;

/**
 * Stellt eine BlueJ-Erweiterung zur Arbeit mit Lego-Robotern zur Verfügung
 * 
 * @author  Frank Gollnik
 * @version 1.1.2
 */
public class XBlueJExtension extends Extension
{
    final XBlueJTools mTools = new XBlueJTools();

    public void startup (BlueJ bluej)
    {
    	if (XBlueJTools.LOGGING)
    	{
    		XBlueJTools.LOGFILE.println("XBlueJExtension.startup(...)");
    		XBlueJTools.LOGFILE.flush();
    	}

    	mTools.startup(bluej);
    }


     /*
     * This method must decide if this Extension is compatible with the 
     * current release of the BlueJ Extensions API
     */
    @Override
    public boolean isCompatible()
    { 
        return (mTools.isCompatible()); 
    }

    /*
     * Returns the version number of this extension
     */
    @Override
    public String getVersion()
    { 
        return (mTools.getVersion());
    }

    /*
     * Returns the user-visible name of this extension
     */
    @Override
    public String getName()
    { 
        return (mTools.getName());  
    }

    @Override
    public void terminate()
    {
        mTools.terminate();
    }
    
    @Override
    public String getDescription()
    {
        return (mTools.getDescription());
    }

    /*
     * Returns a URL where you can find info on this extension.
     * The real problem is making sure that the link will still be alive in three years...
     */
    @Override
    public URL getURL ()
    {
    	return mTools.getURL();
    }
}
