/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.tools;

import java.net.URL;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.File;
import bluej.extensions.*;

/**
 * Implementierungsklasse zur BlueJ-Erweiterung
 * 
 * @author  Frank Gollnik
 * @version 1.1.2
 */
public class XBlueJTools extends XJaboticsTools
{
    BlueJ mBlueJ;
    String mLanguage;

    public void startup(BlueJ bluej)
    {
        if (LOGGING) LOGFILE.println("XBlueJTools.startup(BlueJ bluej)");

        mBlueJ = bluej;
        
        // Initialization ordering:
        // mLanguage -> startup() -> construction of menu and preferences generator

        String language = mBlueJ.getBlueJPropertyString("bluej.language","english");
        if (LOGGING) LOGFILE.println("language = " + language);
        if (language.equals("german"))
        {
            mLanguage = "de";
        }
        else
        {
            mLanguage = "en";
        }

        startup();
        
        // Register a generator for menu items
        bluej.setMenuGenerator(new XMenuGenerator());

        // Register a "preferences" panel generator
        bluej.setPreferenceGenerator(new XPreferences());
    }


    /*
    * This method must decide if this Extension is compatible with the 
    * current release of the BlueJ Extensions API
    */
   public boolean isCompatible()
   { 
       return true; 
   }

    /*
     * Returns the version number of this extension
     */
    public String getVersion()
    { 
        return (VERSION_NUMBER);
    }

    /*
     * Returns the user-visible name of this extension
     */
    public String getName()
    { 
        return (MSG_ROBOT_EXT);  
    }

    public void terminate()
    {
        //System.out.println ("Robot extension terminates");
    }
    
    public String getDescription()
    {
        return (MSG_ROBOT_EXT_INFO);
    }

    /*
     * Returns a URL where you can find info on this extension.
     * The real problem is making sure that the link will still be alive in three years...
     */
    public URL getURL()
    {
        try
        {
            return new URL("http://www.jabotics.org");
        }
        catch ( Exception eee )
        {
            // The link is either dead or otherwise unreachable
            System.out.println (MSG_URL_ERROR + ":\n" + eee.getMessage());
            return null;
        }
    }
    
    @Override
    protected boolean isBlueJ()
    {
        return true;
    }

    @Override
    protected String getLanguage()
    {
        return mLanguage;
    }

    @Override
    protected String getSimLibDirPath()
    {
        return mBlueJ.getSystemLibDir().getAbsolutePath() + File.separatorChar + "userlib";
    }

    @Override
    protected String getJaboticsToolsPath()
    {
        return mBlueJ.getSystemLibDir().getAbsolutePath() + File.separatorChar + "extensions" + File.separatorChar + "jabotics-tools.jar";
    }

    @Override
    protected String getPropertyString(String key, String default_value)
    {
        return mBlueJ.getExtensionPropertyString(key,default_value);
    }

    @Override
    protected void setPropertyString(String key, String value)
    {
        mBlueJ.setExtensionPropertyString(key,value);
    }

    @Override
    protected void showMessageDialog(String message)
    {
        try
        {
            BPackage aPackage = mBlueJ.getCurrentPackage();
            if (null == aPackage)
            {
                JOptionPane.showMessageDialog(mBlueJ.getCurrentFrame(),message);
            }
            else
            {
                JOptionPane.showMessageDialog(aPackage.getFrame(),message);
            }
        }
        catch (HeadlessException | ProjectNotOpenException | PackageNotFoundException e)
        {
            e.printStackTrace();
        }
    }

    @Override
    protected String showInputDialog(String message, String initialSelection)
    {
        return JOptionPane.showInputDialog(message,initialSelection);
    }

    @Override
    protected boolean tryCompileClass(XActionTarget target)
    {
        try
        {
            BClass currentClass = mBlueJ.getCurrentPackage().getCurrentClasses()[0];
            if (!currentClass.getJavaFile().equals(target.mJavaFile)) return false;
            if (currentClass.isCompiled()) return true;
            currentClass.compile(true);
            return currentClass.isCompiled();
        }
        catch (ProjectNotOpenException | PackageNotFoundException | CompilationNotStartedException e)
        {
            e.printStackTrace();
            return false;
        }
    }

    /*
     * This nested class implements the preference panel behaviour for a BlueJ extension
     */
    class XPreferences implements PreferenceGenerator, ActionListener
    {
        private JPanel mPanel;
        private JButton mButtonDetails;
        private JDialog mDialog;
    
        // Construct the panel, and initialize it from any stored values
        //public XPreferences(BlueJ bluej) 
        public XPreferences() 
        {
            if (LOGGING) LOGFILE.println("XPreferences()");

            mPanel = new JPanel(new BorderLayout());
            JTextArea textArea = new JTextArea(mInfo);
            textArea.setEditable(false);
            textArea.setFont(mPanel.getFont());
            textArea.setBackground(mPanel.getBackground());
            mPanel.add(textArea,BorderLayout.CENTER);
            mButtonDetails = new JButton(MSG_SHOW_DETAILS);
            mButtonDetails.addActionListener(this);
            mPanel.add(mButtonDetails,BorderLayout.SOUTH);
        }

        @Override
        public JPanel getPanel ()
        {
            if (LOGGING) LOGFILE.println("XPreferences.getPanel()");
            return mPanel;
        }

        @Override
        public void loadValues () {}

        @Override
        public void saveValues () {}

        @Override
        public void actionPerformed(ActionEvent e)
        {
            if (null == mDialog)
            {
                mDialog = new JDialog(mBlueJ.getCurrentFrame(),MSG_INIT_DETAILS,true);
                JPanel content = new JPanel(new BorderLayout());
                JTextArea textArea = new JTextArea(mInitDetails);
                textArea.setEditable(false);
                JScrollPane scrollPane = new JScrollPane(textArea);
                content.add(scrollPane,BorderLayout.CENTER);
                scrollPane.setPreferredSize(new Dimension(500,300));
                content.add(scrollPane);
                JButton button = new JButton("OK");
                button.addActionListener(this);
                content.add(button,BorderLayout.SOUTH);
                mDialog.add(content);
                mDialog.pack();
                mDialog.setDefaultCloseOperation(WindowConstants.HIDE_ON_CLOSE);
            }

            mDialog.setVisible(e.getSource().equals(mButtonDetails));
        }
    }

    
    /* Diese Klasse generiert die Menüeinträge für die Roboter-Erweiterung
     * Remember:
     * - getToolsMenuItem, getClassMenuItem and getObjectMenuItem may be called by BlueJ at any time.
     * - They must generate a new JMenuItem each time they are called.
     * - No reference to the JMenuItem should be stored in the extension.
     * - You must be quick in generating your menu.
     */
    class XMenuGenerator extends MenuGenerator
    {
        public XMenuGenerator()
        {
            super();
            if (LOGGING) LOGFILE.println("XMenuGenerator()");
        }
    
        @Override
        public JMenuItem getToolsMenuItem(BPackage aPackage)
        {
            if (LOGGING) LOGFILE.println("XMenuGenerator.getToolsMenuItem()");

            XActionTarget target = newActionTarget(aPackage);

            JMenuItem item;
            JMenu menu = new JMenu(MSG_ROBOT);
            menu.setEnabled(mReadyNXT||mReadyEV3);

            item = new JMenuItem(new NXTBrowserAction(target));
            item.setEnabled(mReadyNXT);
            menu.add(item);

            item = new JMenuItem(new NXTConsoleAction(target));
            item.setEnabled(mReadyNXT);
            menu.add(item);

            item = new JMenuItem(new NXTFlashAction(target));
            item.setEnabled(mReadyNXT);
            menu.add(item);

            item = new JMenuItem(new NXTCompilePackageAction(target));
            item.setEnabled(mReadyNXT);
            menu.add(item);

            menu.addSeparator();

            item = new JMenuItem(new EV3SelectRobotAction(target));
            item.setEnabled(mReadyEV3);
            menu.add(item);

            item = new JMenuItem(new EV3ControlAction(target));
            item.setEnabled(mReadyEV3);
            menu.add(item);

            item = new JMenuItem(new EV3SDCardAction(target));
            item.setEnabled(mReadyEV3);
            menu.add(item);

            item = new JMenuItem(new EV3LibsUploadAction(target));
            item.setEnabled(mReadyEV3);
            menu.add(item);

            item = new JMenuItem(new EV3CompilePackageAction(target));
            item.setEnabled(mReadyEV3);
            menu.add(item);

            return menu;
        }
    
        @Override
        public JMenuItem getClassMenuItem(BClass aClass)
        {
            if (LOGGING) LOGFILE.println("XMenuGenerator.getClassMenuItem(...)");

            XActionTarget target = newActionTarget(aClass);

            JMenuItem item;
            JMenu menu = new JMenu(MSG_ROBOT);
            menu.setEnabled(mReadySIM||mReadyNXT||mReadyEV3);

            item = new JMenuItem(new SimulateAction(target));
            item.setEnabled(mReadySIM);
            menu.add(item);

            menu.addSeparator();

            item = new JMenuItem(new NXTCompileAction(target));
            item.setEnabled(mReadyNXT);
            menu.add(item);

            item = new JMenuItem(new NXTUploadAction(target));
            item.setEnabled(mReadyNXT);
            menu.add(item);

            menu.addSeparator();

            item = new JMenuItem(new EV3CompileAction(target));
            item.setEnabled(mReadyEV3);
            menu.add(item);

            item = new JMenuItem(new EV3UploadAction(target));
            item.setEnabled(mReadyEV3 && !mEV3UserlibFiles.isEmpty());
            menu.add(item);

            return menu;
        }

        private XActionTarget newActionTarget(BPackage aPackage)
        {
            try
            {
                BProject aProject = aPackage.getProject();
                File projectDir = aProject.getDir();
                File packageDir = aPackage.getDir();
                String packageName = aPackage.getName();
                return new XActionTarget(projectDir, packageDir, packageName, null, null);
            }
            catch (ProjectNotOpenException | PackageNotFoundException e)
            {
                e.printStackTrace();
                return null;
            }
        }

        private XActionTarget newActionTarget(BClass aClass)
        {
            try
            {
                BPackage aPackage = aClass.getPackage();
                BProject aProject = aPackage.getProject();
                File projectDir = aProject.getDir();
                File packageDir = aPackage.getDir();
                String packageName = aPackage.getName();
                File javaFile = aClass.getJavaFile();
                String className = aClass.getName();
                return new XActionTarget(projectDir, packageDir, packageName, javaFile, className);
            }
            catch (ProjectNotOpenException | PackageNotFoundException e)
            {
                e.printStackTrace();
                return null;
            }
        }
    }
}
